import json
import django
from django.conf import settings
if django.VERSION >= (4, 0):
    from django.db.models import JSONField as DjangoJSONField
else:
    from django.contrib.postgres.fields import JSONField as DjangoJSONField
from django.contrib.postgres.fields import ArrayField as DjangoArrayField
from django.db import models


ORDERING_ACCEPTED_FIELDS = (
    models.BigIntegerField,
    models.IntegerField,
    models.PositiveIntegerField,
    models.PositiveSmallIntegerField,
    models.SmallIntegerField,
)


class JSONField(DjangoJSONField):
    pass


class ArrayField(DjangoArrayField):
    pass


if "sqlite" in settings.DATABASES["default"]["ENGINE"]:

    class JSONField(models.Field):
        def db_type(self, connection):
            return "text"

        def from_db_value(self, value, expression, connection):
            if value is not None:
                return self.to_python(value)
            return value

        def to_python(self, value):
            if value is not None:
                try:
                    return json.loads(value)
                except (TypeError, ValueError):
                    return value
            return value

        def get_prep_value(self, value):
            if value is not None:
                return str(json.dumps(value))
            return value

        def value_to_string(self, obj):
            return self.value_from_object(obj)

    class ArrayField(JSONField):
        def __init__(self, base_field, size=None, **kwargs):
            """Care for DjanroArrayField's kwargs."""
            self.base_field = base_field
            self.size = size
            return super().__init__(**kwargs)

        def deconstruct(self):
            """Need to create migrations properly."""
            name, path, args, kwargs = super().deconstruct()
            kwargs.update(
                {
                    "base_field": self.base_field.clone(),
                    "size": self.size,
                }
            )
            return name, path, args, kwargs



# Generated by Django 3.0 on 2022-08-02 14:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('camomilla', '0009_article__hvad_query_category__hvad_query_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='article',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='articletranslation',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='category',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='categorytranslation',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='content',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='contenttranslation',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='media',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='mediafolder',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='mediafoldertranslation',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='mediatranslation',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='page',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='pagetranslation',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='tag',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='tagtranslation',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



# Generated by Django 4.0.4 on 2022-05-19 17:19

from django.db import migrations, models
import django.db.models.deletion
import hvad.fields


class Migration(migrations.Migration):

    dependencies = [
        ('camomilla', '0008_auto_20220309_1616'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='_hvad_query',
            field=hvad.fields.SingleTranslationObject('camomilla.Article', 'camomilla.articletranslation'),
        ),
        migrations.AddField(
            model_name='category',
            name='_hvad_query',
            field=hvad.fields.SingleTranslationObject('camomilla.Category', 'camomilla.categorytranslation'),
        ),
        migrations.AddField(
            model_name='content',
            name='_hvad_query',
            field=hvad.fields.SingleTranslationObject('camomilla.Content', 'camomilla.contenttranslation'),
        ),
        migrations.AddField(
            model_name='media',
            name='_hvad_query',
            field=hvad.fields.SingleTranslationObject('camomilla.Media', 'camomilla.mediatranslation'),
        ),
        migrations.AddField(
            model_name='mediafolder',
            name='_hvad_query',
            field=hvad.fields.SingleTranslationObject('camomilla.MediaFolder', 'camomilla.mediafoldertranslation'),
        ),
        migrations.AddField(
            model_name='page',
            name='_hvad_query',
            field=hvad.fields.SingleTranslationObject('camomilla.Page', 'camomilla.pagetranslation'),
        ),
        migrations.AddField(
            model_name='tag',
            name='_hvad_query',
            field=hvad.fields.SingleTranslationObject('camomilla.Tag', 'camomilla.tagtranslation'),
        ),
        migrations.AlterField(
            model_name='article',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='article',
            name='og_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(app_label)s_%(class)s_related', to='camomilla.media'),
        ),
        migrations.AlterField(
            model_name='articletranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='articletranslation',
            name='master',
            field=hvad.fields.MasterKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='camomilla.article'),
        ),
        migrations.AlterField(
            model_name='category',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='categorytranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='categorytranslation',
            name='master',
            field=hvad.fields.MasterKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='camomilla.category'),
        ),
        migrations.AlterField(
            model_name='content',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='contenttranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='contenttranslation',
            name='master',
            field=hvad.fields.MasterKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='camomilla.content'),
        ),
        migrations.AlterField(
            model_name='media',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='mediafolder',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='mediafoldertranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='mediafoldertranslation',
            name='master',
            field=hvad.fields.MasterKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='camomilla.mediafolder'),
        ),
        migrations.AlterField(
            model_name='mediatranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='mediatranslation',
            name='master',
            field=hvad.fields.MasterKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='camomilla.media'),
        ),
        migrations.AlterField(
            model_name='page',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='page',
            name='og_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(app_label)s_%(class)s_related', to='camomilla.media'),
        ),
        migrations.AlterField(
            model_name='pagetranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='pagetranslation',
            name='master',
            field=hvad.fields.MasterKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='camomilla.page'),
        ),
        migrations.AlterField(
            model_name='tag',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='tagtranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='tagtranslation',
            name='master',
            field=hvad.fields.MasterKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='camomilla.tag'),
        ),
    ]



from ..mixins import (
    LangInfoMixin,
    SetupEagerLoadingMixin,
    TranslationSetMixin,
    OrderingMixin,
)
from rest_framework import serializers
from hvad.contrib.restframework import TranslatableModelSerializer
from django.db import models
from ..fields import FileField, ImageField, RelatedField
from hvad.models import TranslatableModel


def build_standard_model_serializer(model):
    return type(
        f"{model.__name__}StandardSerializer",
        (
            TranslatableModelSerializer
            if issubclass(model, TranslatableModel)
            else serializers.ModelSerializer,
        ),
        {"Meta": type("Meta", (object,), {"model": model, "fields": "__all__"})},
    )


class BaseModelSerializer(
    OrderingMixin, SetupEagerLoadingMixin, serializers.ModelSerializer
):
    serializer_field_mapping = {
        **serializers.ModelSerializer.serializer_field_mapping,
        models.FileField: FileField,
        models.ImageField: ImageField,
    }
    serializer_related_field = RelatedField

    def build_relational_field(self, field_name, relation_info):
        field_class, field_kwargs = super().build_relational_field(
            field_name, relation_info
        )
        if isinstance(field_class, RelatedField):
            field_kwargs["serializer"] = build_standard_model_serializer(relation_info[1])
        return field_class, field_kwargs


class BaseTranslatableModelSerializer(
    OrderingMixin, LangInfoMixin, TranslationSetMixin, TranslatableModelSerializer
):
    serializer_field_mapping = {
        **TranslatableModelSerializer.serializer_field_mapping,
        models.FileField: FileField,
        models.ImageField: ImageField,
    }
    serializer_related_field = RelatedField

    def build_relational_field(self, field_name, relation_info):
        field_class, field_kwargs = super().build_relational_field(
            field_name, relation_info
        )
        if isinstance(field_class, RelatedField):
            field_kwargs["serializer"] = build_standard_model_serializer(relation_info[1])
        return field_class, field_kwargs
